@@ -74,36 +74,71 @@ namespace ze_lib
             ze_lib::context->zesDdiTable.exchange(&ze_lib::context->initialzesDdiTable);
         }
 
+        // Always call the inits for all the ddi tables before checking which drivers are usable to enable Instrumentation correctly.
+
+        // Init the ZE DDI Tables
         if( ZE_RESULT_SUCCESS == result )
         {
-            result = zeInit();
+            result = zeDdiTableInit();
         }
-
+        // Init the ZET DDI Tables
         if( ZE_RESULT_SUCCESS == result )
         {
-            result = zetInit();
+            result = zetDdiTableInit();
         }
-
+        // Init the ZES DDI Tables
         if( ZE_RESULT_SUCCESS == result )
         {
-            result = zesInit();
+            result = zesDdiTableInit();
         }
-
+        // Init the Tracing API DDI Tables
         if( ZE_RESULT_SUCCESS == result )
         {
-            result = zelTracingInit();
+            result = zelTracingDdiTableInit();
         }
-
+        // Init the stored ddi tables for the tracing layer
         if( ZE_RESULT_SUCCESS == result )
         {
-            result = zelLoaderTracingLayerInit(this->pTracingZeDdiTable, this->pTracingZetDdiTable, this->pTracingZesDdiTable);
+            result = zelLoaderTracingLayerInit(this->pTracingZeDdiTable);
         }
+        // End DDI Table Inits
 
-        if( ZE_RESULT_SUCCESS == result && !sysmanOnly)
+        // Check which drivers and layers can be init on this system.
+        if( ZE_RESULT_SUCCESS == result)
         {
-            //Check which drivers support the ze_driver_flag_t specified
-            //No need to check if only initializing sysman  
-            result = zelLoaderDriverCheck(flags);
+            // Check which drivers support the ze_driver_flag_t specified
+            // No need to check if only initializing sysman
+            bool requireDdiReinit = false;
+            auto sysmanEnv = getenv_tobool( "ZES_ENABLE_SYSMAN" );
+            result = zelLoaderDriverCheck(flags, &ze_lib::context->initialzeDdiTable.Global, &ze_lib::context->initialzesDdiTable.Global, &requireDdiReinit, sysmanOnly);
+            // If a driver was removed from the driver list, then the ddi tables need to be reinit to allow for passthru directly to the driver.
+            // If ZET_ENABLE_PROGRAM_INSTRUMENTATION is enabled, then reInit is not possible due to the functions being intercepted with the previous ddi tables.
+            auto programInstrumentationEnabled = getenv_tobool( "ZET_ENABLE_PROGRAM_INSTRUMENTATION" );
+            if (requireDdiReinit && !programInstrumentationEnabled) {
+                if (!sysmanOnly) {
+                    // reInit the ZE DDI Tables
+                    if( ZE_RESULT_SUCCESS == result )
+                    {
+                        result = zeDdiTableInit();
+                    }
+                    // reInit the ZET DDI Tables
+                    if( ZE_RESULT_SUCCESS == result )
+                    {
+                        result = zetDdiTableInit();
+                    }
+                    // If ze/zet ddi tables have been reinit and no longer use the intercept layer, then handles passed to zelLoaderTranslateHandleInternal do not require translation.
+                    // Setting intercept_enabled==false changes the behavior of zelLoaderTranslateHandleInternal to avoid translation.
+                    // Translation is only required if the intercept layer is enabled for the ZE handle types.
+                    loader::context->intercept_enabled = false;
+                }
+                if (sysmanOnly || sysmanEnv) {
+                    // reInit the ZES DDI Tables
+                    if( ZE_RESULT_SUCCESS == result )
+                    {
+                        result = zesDdiTableInit();
+                    }
+                }
+            }
         }
 
         if( ZE_RESULT_SUCCESS == result )
@@ -162,8 +197,6 @@ zelEnableTracingLayer()
 {
     if (ze_lib::context->tracingLayerEnableCounter.fetch_add(1) == 0) {
         ze_lib::context->zeDdiTable.exchange(ze_lib::context->pTracingZeDdiTable);
-        ze_lib::context->zetDdiTable.exchange(ze_lib::context->pTracingZetDdiTable);
-        ze_lib::context->zesDdiTable.exchange(ze_lib::context->pTracingZesDdiTable);
     }
     return ZE_RESULT_SUCCESS;
 }
@@ -173,8 +206,6 @@ zelDisableTracingLayer()
 {
     if (ze_lib::context->tracingLayerEnableCounter.fetch_sub(1) <= 1) {
         ze_lib::context->zeDdiTable.exchange(&ze_lib::context->initialzeDdiTable);
-        ze_lib::context->zetDdiTable.exchange(&ze_lib::context->initialzetDdiTable);
-        ze_lib::context->zesDdiTable.exchange(&ze_lib::context->initialzesDdiTable);
     }
     return ZE_RESULT_SUCCESS;
 }
