@@ -10,6 +10,12 @@
 #include "driver_discovery.h"
 #include <iostream>
 
+#ifdef __linux__
+#include <unistd.h>
+#include <sys/types.h>
+#include <pwd.h>
+#endif // __linux__
+
 namespace loader
 {
     ///////////////////////////////////////////////////////////////////////////////
@@ -118,38 +124,38 @@ namespace loader
         }
     }
 
-    ze_result_t context_t::check_drivers(ze_init_flags_t flags) {
+    ze_result_t context_t::check_drivers(ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool *requireDdiReinit, bool sysmanOnly) {
         if (debugTraceEnabled) {
             std::string message = "check_drivers(" + std::string("flags=") + loader::to_string(flags) + ")";
             debug_trace_message(message, "");
         }
         bool return_first_driver_result=false;
-        if(drivers.size()==1) {
+        std::string initName = "zeInit";
+        driver_vector_t *drivers = &zeDrivers;
+        // If this is sysmanOnly check_drivers, then zesInit is being called and we need to use zesDrivers.
+        if (sysmanOnly) {
+            drivers = &zesDrivers;
+            initName = "zesInit";
+        }
+        if(drivers->size()==1) {
             return_first_driver_result=true;
         }
 
-        for(auto it = drivers.begin(); it != drivers.end(); )
+        for(auto it = drivers->begin(); it != drivers->end(); )
         {
             std::string freeLibraryErrorValue;
-            ze_result_t result = init_driver(*it, flags);
+            ze_result_t result = init_driver(*it, flags, globalInitStored, sysmanGlobalInitStored, sysmanOnly);
             if(result != ZE_RESULT_SUCCESS) {
-                if (it->handle) {
-                    auto free_result = FREE_DRIVER_LIBRARY(it->handle);
-                    auto failure = FREE_DRIVER_LIBRARY_FAILURE_CHECK(free_result);
-                    if (debugTraceEnabled && failure) {
-                        GET_LIBRARY_ERROR(freeLibraryErrorValue);
-                        if (!freeLibraryErrorValue.empty()) {
-                            std::string errorMessage = "Free Library Failed on " + it->name + " with ";
-                            debug_trace_message(errorMessage, freeLibraryErrorValue);
-                            freeLibraryErrorValue.clear();
-                        }
-                    }
-                }
                 if (debugTraceEnabled) {
-                    std::string errorMessage = "Check Drivers Failed on " + it->name + " , driver will be removed. zeInit failed with ";
+                    std::string errorMessage = "Check Drivers Failed on " + it->name + " , driver will be removed. " + initName + " failed with ";
                     debug_trace_message(errorMessage, loader::to_string(result));
                 }
-                it = drivers.erase(it);
+                it = drivers->erase(it);
+                // If the number of drivers is now ==1, then we need to reinit the ddi tables to pass through.
+                // If ZE_ENABLE_LOADER_INTERCEPT is set to 1, then even if drivers were removed, don't reinit the ddi tables.
+                if (drivers->size() == 1 && !loader::context->forceIntercept) {
+                    *requireDdiReinit = true;
+                }
                 if(return_first_driver_result)
                     return result;
             }
@@ -158,97 +164,104 @@ namespace loader
             }
         }
 
-        if(drivers.size() == 0)
+        if(drivers->size() == 0)
             return ZE_RESULT_ERROR_UNINITIALIZED;
 
         return ZE_RESULT_SUCCESS;
     }
 
-    ze_result_t context_t::init_driver(driver_t driver, ze_init_flags_t flags) {
+    ze_result_t context_t::init_driver(driver_t &driver, ze_init_flags_t flags, ze_global_dditable_t *globalInitStored, zes_global_dditable_t *sysmanGlobalInitStored, bool sysmanOnly) {
 
-        auto getTable = reinterpret_cast<ze_pfnGetGlobalProcAddrTable_t>(
-            GET_FUNCTION_PTR(driver.handle, "zeGetGlobalProcAddrTable"));
-        if(!getTable) {
-            if (debugTraceEnabled) {
-                std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable function pointer null. Returning ";
-                debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
-            }
-            return ZE_RESULT_ERROR_UNINITIALIZED;
-        }
-        
-        ze_global_dditable_t global;
-        auto getTableResult = getTable(ZE_API_VERSION_CURRENT, &global);
-        if(getTableResult != ZE_RESULT_SUCCESS) {
-            if (debugTraceEnabled) {
-                std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable() failed with ";
-                debug_trace_message(errorMessage, loader::to_string(getTableResult));
-            }
-            return ZE_RESULT_ERROR_UNINITIALIZED;
-        }
-        
-        if(nullptr == global.pfnInit) {
-            if (debugTraceEnabled) {
-                std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
-                debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
-            }
-            return ZE_RESULT_ERROR_UNINITIALIZED;
-        }
-
-        if(nullptr != validationLayer) {
-            getTable = reinterpret_cast<ze_pfnGetGlobalProcAddrTable_t>(
-                GET_FUNCTION_PTR(validationLayer, "zeGetGlobalProcAddrTable") );
+        if (sysmanOnly) {
+            auto getTable = reinterpret_cast<zes_pfnGetGlobalProcAddrTable_t>(
+                GET_FUNCTION_PTR(driver.handle, "zesGetGlobalProcAddrTable"));
             if(!getTable) {
                 if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable function pointer null with validation layer. Returning ";
+                    std::string errorMessage = "init driver " + driver.name + " failed, zesGetGlobalProcAddrTable function pointer null. Returning ";
                     debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
                 }
                 return ZE_RESULT_ERROR_UNINITIALIZED;
             }
-            getTableResult = getTable( version, &global);
+
+            zes_global_dditable_t global;
+            auto getTableResult = getTable(ZE_API_VERSION_CURRENT, &global);
             if(getTableResult != ZE_RESULT_SUCCESS) {
                 if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable() with validation layer failed with ";
+                    std::string errorMessage = "init driver " + driver.name + " failed, zesGetGlobalProcAddrTable() failed with ";
+                    debug_trace_message(errorMessage, loader::to_string(getTableResult));
+                }
+                return ZE_RESULT_ERROR_UNINITIALIZED;
+            }
+
+            if(nullptr == global.pfnInit) {
+                if (debugTraceEnabled) {
+                    std::string errorMessage = "init driver " + driver.name + " failed, zesInit function pointer null. Returning ";
                     debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
                 }
                 return ZE_RESULT_ERROR_UNINITIALIZED;
             }
-        }
 
-        if(nullptr != tracingLayer) {
-            getTable = reinterpret_cast<ze_pfnGetGlobalProcAddrTable_t>(
-                GET_FUNCTION_PTR(tracingLayer, "zeGetGlobalProcAddrTable") );
+            // Use the previously init ddi table pointer to zesInit to allow for intercept of the zesInit calls
+            ze_result_t res = sysmanGlobalInitStored->pfnInit(flags);
+            // Verify that this driver successfully init in the call above.
+            if (driver.initStatus != ZE_RESULT_SUCCESS) {
+                res = driver.initStatus;
+            }
+            if (debugTraceEnabled) {
+                std::string message = "init driver " + driver.name + " zesInit(" + loader::to_string(flags) + ") returning ";
+                debug_trace_message(message, loader::to_string(res));
+            }
+            return res;
+        } else {
+            auto getTable = reinterpret_cast<ze_pfnGetGlobalProcAddrTable_t>(
+                GET_FUNCTION_PTR(driver.handle, "zeGetGlobalProcAddrTable"));
             if(!getTable) {
                 if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable function pointer null with tracing layer. Returning ";
+                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable function pointer null. Returning ";
                     debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
                 }
                 return ZE_RESULT_ERROR_UNINITIALIZED;
             }
-            getTableResult = getTable( version, &global);
+
+            ze_global_dditable_t global;
+            auto getTableResult = getTable(ZE_API_VERSION_CURRENT, &global);
             if(getTableResult != ZE_RESULT_SUCCESS) {
                 if (debugTraceEnabled) {
-                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable() with tracing layer failed with ";
+                    std::string errorMessage = "init driver " + driver.name + " failed, zeGetGlobalProcAddrTable() failed with ";
+                    debug_trace_message(errorMessage, loader::to_string(getTableResult));
+                }
+                return ZE_RESULT_ERROR_UNINITIALIZED;
+            }
+
+            if(nullptr == global.pfnInit) {
+                if (debugTraceEnabled) {
+                    std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
                     debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
                 }
                 return ZE_RESULT_ERROR_UNINITIALIZED;
             }
-        }
 
-        auto pfnInit = global.pfnInit;
-        if(nullptr == pfnInit) {
-            if (debugTraceEnabled) {
-                std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
-                debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+            auto pfnInit = global.pfnInit;
+            if(nullptr == pfnInit || globalInitStored->pfnInit == nullptr) {
+                if (debugTraceEnabled) {
+                    std::string errorMessage = "init driver " + driver.name + " failed, zeInit function pointer null. Returning ";
+                    debug_trace_message(errorMessage, loader::to_string(ZE_RESULT_ERROR_UNINITIALIZED));
+                }
+                return ZE_RESULT_ERROR_UNINITIALIZED;
             }
-            return ZE_RESULT_ERROR_UNINITIALIZED;
-        }
 
-        ze_result_t res = pfnInit(flags);
-        if (debugTraceEnabled) {
-            std::string message = "init driver " + driver.name + " zeInit(" + loader::to_string(flags) + ") returning ";
-            debug_trace_message(message, loader::to_string(res));
+            // Use the previously init ddi table pointer to zeInit to allow for intercept of the zeInit calls
+            ze_result_t res = globalInitStored->pfnInit(flags);
+            // Verify that this driver successfully init in the call above.
+            if (driver.initStatus != ZE_RESULT_SUCCESS) {
+                res = driver.initStatus;
+            }
+            if (debugTraceEnabled) {
+                std::string message = "init driver " + driver.name + " zeInit(" + loader::to_string(flags) + ") returning ";
+                debug_trace_message(message, loader::to_string(res));
+            }
+            return res;
         }
-        return res;
     }
 
     ///////////////////////////////////////////////////////////////////////////////
@@ -261,16 +274,49 @@ namespace loader
         auto discoveredDrivers = discoverEnabledDrivers();
         std::string loadLibraryErrorValue;
 
+        auto log_directory = getenv_string("ZEL_LOADER_LOG_DIR");
+        if (log_directory.empty()) {
+            std::string home_dir;
+#ifdef _WIN32
+            home_dir = getenv_string("USERPROFILE");
+            if (home_dir == ""){
+                auto home_drive = getenv_string("HOMEDRIVE");
+                auto home_path = getenv_string("HOMEPATH");
+                if ((home_drive != "") && (home_path != "")) {
+                    home_dir = home_drive + home_path;
+                } else {
+                    home_dir = ".";
+                }
+            }
+            log_directory = home_dir + "\\" + LOADER_LOG_FILE_DIRECTORY;
+#else
+            home_dir = getenv_string("HOME");
 
+            if (home_dir == "") {
+                auto pwdir = getpwuid(getuid())->pw_dir;
+                home_dir = (pwdir == NULL) ? "." : std::string(pwdir);
+            }
+            log_directory = home_dir + "/" + LOADER_LOG_FILE_DIRECTORY;
+#endif
+        }
         auto loader_file = getenv_string("ZEL_LOADER_LOG_FILE");
         if (loader_file.empty()){
-            loader_file = LOADER_LOG_FILE_DEFAULT;
+            loader_file = LOADER_LOG_FILE;
+        } else {
+            auto log_depr_msg = "ZEL_LOADER_LOG_FILE will be deprecated in a future release";
+            std::cout << log_depr_msg << std::endl;
         }
 
+        std::string full_log_file_path = "";
+#ifdef _WIN32
+        full_log_file_path = log_directory + "\\" + loader_file;
+#else
+        full_log_file_path = log_directory + "/" + loader_file;
+#endif
         auto logging_enabled = getenv_tobool( "ZEL_ENABLE_LOADER_LOGGING" );
 
         auto log_level = getenv_string("ZEL_LOADER_LOGGING_LEVEL");
-        zel_logger =  std::make_shared<Logger>("ze_loader", loader_file, !log_level.empty() ? log_level : "warn", logging_enabled);
+        zel_logger =  std::make_shared<Logger>("ze_loader", full_log_file_path, !log_level.empty() ? log_level : "warn", logging_enabled);
         if ((log_level == "trace") && !debugTraceEnabled) {
             debugTraceEnabled = true;
             zel_logger->log_to_console = false;
@@ -283,24 +329,40 @@ namespace loader
         if (zel_logger->logging_enabled)
             zel_logger->get_base_logger()->info("Loader Version {}.{}.{} {}", LOADER_VERSION_MAJOR, LOADER_VERSION_MINOR, LOADER_VERSION_PATCH, LOADER_VERSION_SHA);
 
+        add_loader_version();
+        std::string loaderLibraryPath;
+        auto loaderLibraryPathEnv = getenv_string("ZEL_LIBRARY_PATH");
+        if (!loaderLibraryPathEnv.empty()) {
+            loaderLibraryPath = loaderLibraryPathEnv;
+        }
+#ifdef _WIN32
+        else {
+            loaderLibraryPath = readLevelZeroLoaderLibraryPath();
+        }
+#endif
+        debug_trace_message("Using Loader Library Path: ", loaderLibraryPath);
 
-        drivers.reserve( discoveredDrivers.size() + getenv_tobool( "ZE_ENABLE_NULL_DRIVER" ) );
+        // To allow for two different sets of drivers to be in use between sysman and core/tools, we use and store the drivers in two vectors.
+        // alldrivers stores all the drivers for cleanup when the library exits.
+        zeDrivers.reserve( discoveredDrivers.size() + getenv_tobool( "ZE_ENABLE_NULL_DRIVER" ) );
+        zesDrivers.reserve( discoveredDrivers.size() + getenv_tobool( "ZE_ENABLE_NULL_DRIVER" ) );
+        allDrivers.reserve( discoveredDrivers.size() + getenv_tobool( "ZE_ENABLE_NULL_DRIVER" ) );
         if( getenv_tobool( "ZE_ENABLE_NULL_DRIVER" ) )
         {
             zel_logger->log_info("Enabling Null Driver");
-            auto handle = LOAD_DRIVER_LIBRARY( MAKE_LIBRARY_NAME( "ze_null", L0_LOADER_VERSION ) );
+            auto handle = LOAD_DRIVER_LIBRARY( create_library_path( MAKE_LIBRARY_NAME( "ze_null", L0_LOADER_VERSION ), loaderLibraryPath.c_str()).c_str());
             if (debugTraceEnabled) {
                 std::string message = "ze_null Driver Init";
                 debug_trace_message(message, "");
             }
             if( NULL != handle )
             {
-                drivers.emplace_back();
-                drivers.rbegin()->handle = handle;
-                drivers.rbegin()->name = "ze_null";
+                allDrivers.emplace_back();
+                allDrivers.rbegin()->handle = handle;
+                allDrivers.rbegin()->name = "ze_null";
             } else if (debugTraceEnabled) {
                 GET_LIBRARY_ERROR(loadLibraryErrorValue);
-                std::string errorMessage = "Load Library of " + std::string(MAKE_LIBRARY_NAME( "ze_null", L0_LOADER_VERSION )) + " failed with ";
+                std::string errorMessage = "Load Library of " + create_library_path( MAKE_LIBRARY_NAME( "ze_null", L0_LOADER_VERSION ), loaderLibraryPath.c_str()) + " failed with ";
                 debug_trace_message(errorMessage, loadLibraryErrorValue);
                 loadLibraryErrorValue.clear();
             }
@@ -315,9 +377,9 @@ namespace loader
             }
             if( NULL != handle )
             {
-                drivers.emplace_back();
-                drivers.rbegin()->handle = handle;
-                drivers.rbegin()->name = name;
+                allDrivers.emplace_back();
+                allDrivers.rbegin()->handle = handle;
+                allDrivers.rbegin()->name = name;
             } else if (debugTraceEnabled) {
                 GET_LIBRARY_ERROR(loadLibraryErrorValue);
                 std::string errorMessage = "Load Library of " + name + " failed with ";
@@ -325,16 +387,15 @@ namespace loader
                 loadLibraryErrorValue.clear();
             }
         }
-        if(drivers.size()==0){
+        if(allDrivers.size()==0){
+            std::string message = "0 Drivers Discovered";
+            debug_trace_message(message, "");
             zel_logger->log_error("0 Drivers Discovered");
             return ZE_RESULT_ERROR_UNINITIALIZED;
         }
+        std::copy(allDrivers.begin(), allDrivers.end(), std::back_inserter(zeDrivers));
+        std::copy(allDrivers.begin(), allDrivers.end(), std::back_inserter(zesDrivers));
 
-        add_loader_version();
-        std::string loaderLibraryPath;
-#ifdef _WIN32
-        loaderLibraryPath = readLevelZeroLoaderLibraryPath();
-#endif
         typedef ze_result_t (ZE_APICALL *getVersion_t)(zel_component_version_t *version);
         if( getenv_tobool( "ZE_ENABLE_VALIDATION_LAYER" ) )
         {
@@ -363,6 +424,7 @@ namespace loader
             tracingLayerEnabled = true;
         }
         std::string tracingLayerLibraryPath = create_library_path(MAKE_LAYER_NAME( "ze_tracing_layer" ), loaderLibraryPath.c_str());
+        debug_trace_message("Tracing Layer Library Path: ", tracingLayerLibraryPath);
         tracingLayer = LOAD_DRIVER_LIBRARY( tracingLayerLibraryPath.c_str() );
         if(tracingLayer)
         {
@@ -387,8 +449,14 @@ namespace loader
         }
 
         forceIntercept = getenv_tobool( "ZE_ENABLE_LOADER_INTERCEPT" );
+        auto sysmanEnv = getenv_tobool( "ZES_ENABLE_SYSMAN" );
 
-        if(forceIntercept || drivers.size() > 1){
+        sysmanInstanceDrivers = &zesDrivers;
+        if (sysmanEnv) {
+            sysmanInstanceDrivers = &zeDrivers;
+        }
+
+        if(forceIntercept || allDrivers.size() > 1){
              intercept_enabled = true;
              zel_logger->log_info("Intercept Enabled");
         }
@@ -428,7 +496,7 @@ namespace loader
             }
         }
 
-        for( auto& drv : drivers )
+        for( auto& drv : allDrivers )
         {
             if (drv.handle) {
                 auto free_result = FREE_DRIVER_LIBRARY( drv.handle );
